@@ -16,26 +16,24 @@ import logging
 import os
 import sys
 import time
+from configparser import ConfigParser
 from datetime import datetime
-from urllib.request import urlopen
 
 import boto3
 from botocore.config import Config
 from botocore.exceptions import ClientError
-from configparser import ConfigParser
-from retrying import RetryError, retry
-
 from common.time_utils import minutes, seconds
 from common.utils import (
-    CriticalError,
     get_asg_name,
     get_asg_settings,
     get_instance_properties,
+    get_metadata,
     load_additional_instance_types_data,
     load_module,
     retrieve_max_cluster_size,
     sleep_remaining_loop_time,
 )
+from retrying import RetryError, retry
 
 log = logging.getLogger(__name__)
 
@@ -93,24 +91,6 @@ def _get_config():
     return NodewatcherConfig(region, scheduler, stack_name, scaledown_idletime, proxy_config, instance_types_data)
 
 
-def _get_metadata(metadata_path):
-    """
-    Get EC2 instance metadata.
-
-    :param metadata_path: the metadata relative path
-    :return the metadata value.
-    """
-    try:
-        metadata_value = urlopen("http://169.254.169.254/latest/meta-data/{0}".format(metadata_path)).read().decode()
-    except Exception as e:
-        error_msg = "Unable to get {0} metadata. Failed with exception: {1}".format(metadata_path, e)
-        log.critical(error_msg)
-        raise CriticalError(error_msg)
-
-    log.debug("%s=%s", metadata_path, metadata_value)
-    return metadata_value
-
-
 def _has_jobs(scheduler_module, hostname):
     """
     Verify if there are running jobs in the given host.
@@ -390,9 +370,9 @@ def main():
 
         scheduler_module = load_module("nodewatcher.plugins." + config.scheduler)
 
-        instance_id = _get_metadata("instance-id")
-        hostname = _get_metadata("local-hostname")
-        instance_type = _get_metadata("instance-type")
+        instance_id = get_metadata("instance-id")
+        hostname = get_metadata("local-hostname")
+        instance_type = get_metadata("instance-type")
         log.info("Instance id is %s, hostname is %s, instance type is %s", instance_id, hostname, instance_type)
         asg_name = get_asg_name(config.stack_name, config.region, config.proxy_config)
 
